/*
** a_alloc.c - Allocation routines that either succeeds or abort.
**
** Copyright (c) 1997-2000 Peter Eriksson <pen@lysator.liu.se>
**
** This program is free software; you can redistribute it and/or
** modify it as you wish - as long as you don't claim that you wrote
** it.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#include "plib/config.h"

#include <stdlib.h>
#include <syslog.h>
#include <string.h>

#include "plib/safeio.h"

#include "plib/aalloc.h"


/*
** A "safe" malloc, that always succeeds (or logs an
** error to syslog and then abort()'s.
**
** The buffer returned is zeroed out.
*/
void *
a_malloc(size_t size)
{
    void *p;

    
    p = (void *) malloc(size);
    if (p == NULL)
    {
	syslog(LOG_ERR, "a_malloc: malloc(%lu): %m", (unsigned long) size);
	s_abort();
    }

    memset(p, 0, size);
    return p;
}


void *
a_realloc(void *oldp, size_t size)
{
    void *p;

    
    if (oldp)
    {
	p = (void *) realloc(oldp, size);
	if (p == NULL)
	{
	    syslog(LOG_ERR, "a_realloc: realloc(...,%lu): %m",
		   (unsigned long) size);
	    s_abort();
	}
    }
    else
    {
	p = (void *) malloc(size);
	if (p == NULL)
	{
	    syslog(LOG_ERR, "a_realloc: malloc(%lu): %m",
		   (unsigned long) size);
	    s_abort();
	}
    }
    return p;
}


void
a_free(void *p)
{
    if (p != NULL)
	free(p);
}



char *
a_strndup(const char *s,
	  size_t len)
{
    char *ns;
    
    
    if (s == NULL)
	return NULL;

    ns = (char *) malloc(len+1);
    if (ns == NULL)
    {
	syslog(LOG_ERR, "a_strndup(): malloc(%lu): %m",
	       (unsigned long) len+1);
	s_abort();
    }

    memcpy(ns, s, len);
    ns[len] = '\0';
    return ns;
}


char *
a_strdup(const char *s)
{
    if (s)
	return a_strndup(s, strlen(s));
    else
	return NULL;
}

